﻿using System.ComponentModel.Composition;
using GalaSoft.MvvmLight.Messaging;
using Microsoft.Office.Interop.Word;

namespace Songhay.Office2010.WordWalkingStick.Commands
{
    using Songhay.Models;
    using Songhay.Office2010.Word;

    /// <summary>
    /// Gets Flat OPC output from Word to send
    /// in a message for the application domain.
    /// </summary>
    [Export(typeof(IOfficeWordCommand))]
    public class GetFlatOpcCommand : IOfficeWordCommand
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="GetFlatOpcCommand"/> class.
        /// </summary>
        public GetFlatOpcCommand()
        {
            this.Name = "GetFlatOpc";
        }

        #region IOfficeWordCommand Members

        /// <summary>
        /// Gets or sets the name
        /// (non-unique key)
        /// for locating this command
        /// in a collection of commands.
        /// </summary>
        /// <value>The name.</value>
        /// <remarks>
        /// Conventional command name prefixes:
        /// Startup* Commands with this prefix will be executed during the ThisAddIn_Startup event.
        /// Shutdown* Commands with this prefix will be executed during the ThisAddIn_Shutdown event.
        /// </remarks>
        public string Name { get; set; }

        /// <summary>
        /// Executes the command.
        /// </summary>
        /// <typeparam name="TMessage">The type of the message.</typeparam>
        /// <param name="wordApplication">The word application.</param>
        /// <param name="message">The message.</param>
        public void ExecuteCommand<TMessage>(Application wordApplication, TMessage message) where TMessage: IControlInteractionMessage
        {
            var doc = wordApplication.ActiveDocument;
            var range = RangeUtility.GetRange(doc);

            var viewModelMessage = new TransformationMessage(message)
            {
                TransformationResult = range.WordOpenXML
            };

            //Send MvvmLight message:
            var gmessage = new GenericMessage<TransformationMessage>(viewModelMessage);
            Messenger.Default.Send(gmessage);
        }

        /// <summary>
        /// Executes the command for binding.
        /// </summary>
        /// <typeparam name="TMessage">The type of the message.</typeparam>
        /// <param name="wordApplication">The word application.</param>
        /// <param name="message">The message.</param>
        public object ExecuteCommandForBinding<TMessage>(Application wordApplication, TMessage message) where TMessage : IControlInteractionMessage
        {
            throw new System.NotImplementedException();
        }

        #endregion
    }
}
