﻿using System;
using System.IO;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Songhay.Tests
{
    using Extensions;
    using Songhay;

    /// <summary>
    /// Summary description for UriTest
    /// </summary>
    [TestClass]
    public class OpcUtilityTest
    {
        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext { get; set; }

        [TestInitialize]
        public void ClearPreviousTestResults()
        {
            var directory = Directory.GetParent(TestContext.TestDir);

            directory.GetFiles()
                .OrderByDescending(f => f.LastAccessTime).Skip(1)
                .ForEachInEnumerable(f => f.Delete());

            directory.GetDirectories()
                .OrderByDescending(d => d.LastAccessTime).Skip(1)
                .ForEachInEnumerable(d => d.Delete(true));
        }

        /// <summary>
        /// Should assert that the pack URI is valid.
        /// </summary>
        /// <remarks>
        /// For more detail, see:
        ///     “Pack URIs in WPF” (http://msdn.microsoft.com/en-us/library/aa970069.aspx)
        ///     “The "pack" URI Scheme” (http://tools.ietf.org/html/draft-shur-pack-uri-scheme-05)
        /// </remarks>
        [TestMethod]
        [Description("Should assert Pack URI is valid.")]
        public void ShouldAssertPackUriIsValid()
        {
            var uri = "pack://application:,,,/ReferencedAssembly;component/Subfolder/ResourceFile.xaml";
            var t = new Uri(uri);
            var result = OpcUtility.PackUriIsValid(uri);

            Assert.IsTrue(result, "The valid Pack URI is not here.");
        }

        [TestMethod]
        [Description("Should be a relative URI.")]
        public void ShouldBeARelativeUri()
        {
            var uri = "Foo";
            var result = Uri.IsWellFormedUriString(uri, UriKind.Relative);
            Assert.IsTrue(result, "This is not a valid relative URI");

            uri = "pack://application:,,,/ReferencedAssembly;component/Subfolder/ResourceFile.xaml";
            var relativeUri = OpcUtility.GetRelativePackUri(uri);
        }

        [TestMethod]
        [Description("Should get Pack URI from Type.")]
        public void ShouldGetPackUriFromType()
        {
            var expected = "pack://application:,,,/Songhay.Tests;component/Subfolder/UriTest.xaml";

            var actual = OpcUtility.GetPackUriFromType(this.GetType(),
                OpcReferencedTypeStrategy.FromAssemblyFileName);

            Assert.AreEqual(expected, actual, "The expected pack URI is not here.");

            actual = OpcUtility.GetPackUriFromType(this.GetType(),
                OpcReferencedTypeStrategy.FromTypeFullName);

            Assert.AreEqual(expected, actual, "The expected pack URI is not here.");
        }
    }
}
;