﻿using System;
using System.Globalization;
using System.Windows;
using System.Windows.Data;

namespace Songhay.ValueConverters
{
    /// <summary>
    /// Converts <see cref="System.Boolean"/>
    /// to/from <see cref="System.Windows.Visibility"/> data.
    /// </summary>
    /// <remarks>
    /// For more, see “How do I invert BooleanToVisibilityConverter?”
    /// [http://stackoverflow.com/questions/534575/how-do-i-invert-booleantovisibilityconverter]
    /// </remarks>
    public sealed class BooleanToVisibilityConverter : IValueConverter
    {
        /// <summary>
        /// Converts a value.
        /// </summary>
        /// <param name="value">The value produced by the binding source.</param>
        /// <param name="targetType">The type of the binding target property.</param>
        /// <param name="parameter">The converter parameter to use.</param>
        /// <param name="culture">The culture to use in the converter.</param>
        /// <returns>
        /// A converted value. If the method returns null, the valid null value is used.
        /// </returns>
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            var flag = false;
            if(value is bool)
            {
                flag = (bool)value;
            }
            else if(value is bool?)
            {
                var nullable = (bool?)value;
                flag = nullable.GetValueOrDefault();
            }
            if(parameter != null)
            {
                if(bool.Parse((string)parameter))
                {
                    flag = !flag;
                }
            }
            if(flag)
            {
                return Visibility.Visible;
            }
            else
            {
                return Visibility.Collapsed;
            }
        }

        /// <summary>
        /// Converts a value.
        /// </summary>
        /// <param name="value">The value that is produced by the binding target.</param>
        /// <param name="targetType">The type to convert to.</param>
        /// <param name="parameter">The converter parameter to use.</param>
        /// <param name="culture">The culture to use in the converter.</param>
        /// <returns>
        /// A converted value. If the method returns null, the valid null value is used.
        /// </returns>
        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            var back = ((value is Visibility) && (((Visibility)value) == Visibility.Visible));
            if(parameter != null)
            {
                if((bool)parameter)
                {
                    back = !back;
                }
            }
            return back;
        }
    }
}
