﻿
using System;
using System.Xml.Schema;
using System.Xml.Serialization;

namespace Songhay.Models
{
    /// <summary>
    /// Dave Winer, his Outline Processor Markup Language document format
    /// </summary>
    /// <remarks>
    /// “OPML an XML-based format that allows exchange of outline-structured information
    /// between applications running on different operating systems and environments.”
    /// http://www.opml.org/about
    /// </remarks>
#if !SILVERLIGHT
    [Serializable]
#endif
    [XmlRoot(ElementName = "opml", Namespace = "http://songhaysystem.com/schemas/opml.xsd")]
    public class OpmlDocument
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="OpmlDocument"/> class.
        /// </summary>
        public OpmlDocument()
        {
            this.Version = "2.0";
            this.XsiSchemaLocation = "http://songhaysystem.com/schemas/opml.xsd " +
                "http://songhaysystem.com/schemas/opml.xsd";
        }

        /// <summary>
        /// Gets or sets the schema location.
        /// </summary>
        /// <value>The schema location.</value>
#if SILVERLIGHT
        [XmlAttribute("schemaLocation", Namespace = "http://songhaysystem.com/schemas/opml.xsd")]
#else
        [XmlAttribute("schemaLocation", Namespace = XmlSchema.InstanceNamespace)]
#endif
        public string XsiSchemaLocation { get; set; }

        /// <summary>
        /// Gets the version.
        /// </summary>
        /// <value>The version.</value>
        [XmlAttribute(AttributeName = "version")]
        public string Version { get; set; }

        /// <summary>
        /// Gets the OPML head element.
        /// </summary>
        /// <value>The OPML head element.</value>
        [XmlElement(ElementName = "head")]
        public OpmlHead OpmlHead { get; set; }

        /// <summary>
        /// Gets the OPML body element.
        /// </summary>
        /// <value>The OPML body element.</value>
        [XmlElement(ElementName = "body")]
        public OpmlBody OpmlBody { get; set; }
    }
}
