﻿using System.Collections;
using System.Globalization;
using System.Linq;
using System.Xml;
using System.Xml.Linq;
using System.Xml.XPath;

namespace Songhay.Xml
{
    /// <summary>
    /// Static helper members for XML-related routines.
    /// </summary>
    /// <remarks>
    /// These definitions are biased toward
    /// emitting <see cref="System.Xml.XPath.XPathDocument"/> sets.
    /// However, many accept any input implementing the
    /// <see cref="System.Xml.XPath.IXPathNavigable"/> interface.
    /// </remarks>
    public static partial class XObjectUtility
    {
        /// <summary>
        /// Gets the XObject.
        /// </summary>
        /// <param name="set">The <see cref="System.Xml.Linq.XNode"/> set.</param>
        /// <param name="setQuery">The <see cref="System.String"/></param>
        public static XObject GetXObject(XNode set, string setQuery)
        {
            return GetXObject(set, setQuery, false, null);
        }

        /// <summary>
        /// Gets the XObject.
        /// </summary>
        /// <param name="set">The <see cref="System.Xml.Linq.XNode"/> set.</param>
        /// <param name="setQuery">The <see cref="System.String"/></param>
        /// <param name="throwException">When <code>true</code>, throw an exception for null nodes.</param>
        public static XObject GetXObject(XNode set, string setQuery, bool throwException)
        {
            return GetXObject(set, setQuery, throwException, null);
        }

        /// <summary>
        /// Gets the XObject.
        /// </summary>
        /// <param name="set">The <see cref="System.Xml.Linq.XNode"/> set.</param>
        /// <param name="setQuery">The <see cref="System.String"/></param>
        /// <param name="throwException">When <code>true</code>, throw an exception for null nodes.</param>
        /// <param name="resolver">
        /// The <see cref="System.Xml.IXmlNamespaceResolver"/>
        /// to use to resolve prefixes.
        /// </param>
        public static XObject GetXObject(XNode set, string setQuery, bool throwException, IXmlNamespaceResolver resolver)
        {
            if(set == null) return null;

            var o = (resolver == null)
                ? ((IEnumerable)set.XPathEvaluate(setQuery)).Cast<XObject>().FirstOrDefault()
                : ((IEnumerable)set.XPathEvaluate(setQuery, resolver)).Cast<XObject>().FirstOrDefault();
            if(o != null)
            {
                return o;
            }
            else if(throwException)
            {
                throw new XmlException(string.Format(CultureInfo.CurrentCulture, "Element at “{0}” was not found.", setQuery));
            }
            else
            {
                return null;
            }
        }
    }
}
